@@ -725,12 +725,23 @@ def test_list_comprehension(nested):
 def test_list_comprehension_more(nested):
     """more tests of list comprehension"""
     interp = make_interpreter(nested_symtable=nested)
+    odd = [1, 3, 5, 7, 9, 11, 13, 15, 17, 19]
+    even = [2, 4, 6, 8, 10, 12, 14, 16, 18, 20]
+
+    interp('odd = [1, 3, 5, 7, 9, 11, 13, 15, 17, 19]')
+    interp('even = [2, 4, 6, 8, 10, 12, 14, 16, 18, 20]')
 
     for expr in  ['[2.5*x for x in range(4)]',
                 '[(i, 5*i+j) for i in range(6) for j in range(3)]',
                 '[(i, j*2) for i in range(6) for j in range(2) if i*j < 8]',
                 '[(x, y) for (x,y) in [(1,2), (3,4)]]',
-                '[(2*x, x+y) for (x,y) in [(1,3), (5,9)]]' ]:
+                '[(2*x, x+y) for (x,y) in [(1,3), (5,9)]]',
+                '[p*2.5 for p in odd]',
+                '[n for p in zip(odd, even) for n in p]',
+                '[(i*i + 0.5) for i in range(4)]',
+                '[i*3.2 for i in odd if i > 6 and i < 18]',
+                '[i-1.0 for i in odd if i > 4 and i*2 not in (26, 34)]',
+                ]:
 
         interp(f"out = {expr}")
         result = interp.symtable.get('out')
@@ -1094,9 +1105,9 @@ def test_astdump(nested):
     assert isinstance(astnode, ast.Module)
     assert isinstance(astnode.body[0], ast.Assign)
     assert isinstance(astnode.body[0].targets[0], ast.Name)
-    assert isinstance(astnode.body[0].value, ast.Num)
+    assert isinstance(astnode.body[0].value, ast.Constant)
     assert astnode.body[0].targets[0].id == 'x'
-    assert astnode.body[0].value.n == 1
+    assert astnode.body[0].value.value == 1
     dumped = interp.dump(astnode.body[0])
     assert dumped.startswith('Assign')
 
